/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.core.client.util;

/**
 * Represents a double precision point on the (x, y) coordinate plane.
 */
public class PrecisePoint {

  private double x = 0;
  private double y = 0;

  /**
   * Creates a double precision point.
   */
  public PrecisePoint() {
  }

  /**
   * Creates a double precision point with the given x and y coordinates.
   * 
   * @param x the x coordinate of the point
   * @param y the y coordinate of the point
   */
  public PrecisePoint(double x, double y) {
    this.x = x;
    this.y = y;
  }

  /**
   * Creates a copy of the given {@link PrecisePoint}.
   * 
   * @param point the point to be copied
   */
  public PrecisePoint(PrecisePoint point) {
    this.x = point.getX();
    this.y = point.getY();
  }

  @Override
  public boolean equals(Object obj) {
    if (obj instanceof PrecisePoint) {
      PrecisePoint point = (PrecisePoint) obj;
      return (Double.compare(this.getX(), point.getX()) == 0 && Double.compare(this.getY(), point.getY()) == 0);
    }
    return super.equals(obj);
  }

  /**
   * Indicates whether or not the given object is equal to the rounded values of
   * this {@link PrecisePoint}.
   * 
   * @param obj the reference object with which to compare
   * @param tolerance the tolerance of the difference between the points
   * @return true if this object is the same as the obj argument; false
   *         otherwise
   */
  public boolean equalsNoPrecision(Object obj, double tolerance) {
    if (obj instanceof PrecisePoint) {
      PrecisePoint point = (PrecisePoint) obj;
      long diffX = Math.round(this.getX() - point.getX());
      long diffY = Math.round(this.getY() - point.getY());
      return (diffX >= -tolerance && diffX <= tolerance 
      && diffY >= -tolerance && diffY <= tolerance);
    }
    return super.equals(obj);

  }

  /**
   * Returns the x coordinate of the point.
   * 
   * @return the x coordinate
   */
  public double getX() {
    return x;
  }

  /**
   * Returns the y coordinate of the point.
   * 
   * @return the y coordinate
   */
  public double getY() {
    return y;
  }

  /**
   * Sets the x-coordinate.
   * 
   * @param x the x-coordinate
   */
  public void setX(double x) {
    this.x = x;
  }

  /**
   * Sets the y-coordinate
   * 
   * @param y the y-coordinate
   */
  public void setY(double y) {
    this.y = y;
  }

  @Override
  public String toString() {
    return "(" + x + ", " + y + ")";
  }

}
