/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.data.client.loader;

import com.google.gwt.core.client.Callback;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.jsonp.client.JsonpRequestBuilder;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.sencha.gxt.data.shared.loader.DataProxy;
import com.sencha.gxt.data.shared.writer.DataWriter;

/**
 * A <code>DataProxy</code> that reads a data from a URL which may be in a
 * domain other than the originating domain of the running page.
 * 
 * <p />
 * Note that if you are retrieving data from a page that is in a domain that is
 * NOT the same as the originating domain of the running page, you must use this
 * class, rather than HttpProxy.
 * 
 * <p />
 * When using a load config object that implements <code>LoadConfig</code> or
 * <code>ModelData</code>, all properties and property values will be sent as
 * request parameters in the load request.
 * 
 * 
 * @see HttpProxy
 */
public class ScriptTagProxy<C> implements DataProxy<C, JavaScriptObject> {

  private static int ID = 0;
  private String url;

  private DataWriter<C, String> writer;

  public ScriptTagProxy(String url) {
    this.url = url;
  }

  public DataWriter<C, String> getWriter() {
    return writer;
  }

  public void load(C loadConfig, final Callback<JavaScriptObject, Throwable> callback) {
    String transId = "transId" + ID++;
    String prepend = url.indexOf("?") != -1 ? "&" : "?";
    String u = url + prepend + generateUrl(loadConfig);

    JsonpRequestBuilder b = new JsonpRequestBuilder();
    b.setPredeterminedId(transId);// needed?
    b.requestObject(u, new AsyncCallback<JavaScriptObject>() {
      @Override
      public void onFailure(Throwable caught) {
        callback.onFailure(caught);
      }

      @Override
      public void onSuccess(JavaScriptObject result) {
        callback.onSuccess(result);
      }
    });
  }

  /**
   * Sets the proxies url.
   * 
   * @param url the url
   */
  public void setUrl(String url) {
    this.url = url;
  }

  public void setWriter(DataWriter<C, String> writer) {
    this.writer = writer;
  }

  protected String generateUrl(C loadConfig) {
    if (writer != null) {
      return writer.write(loadConfig);
    } else {
      if (loadConfig == null) {
        return "";
      }
      return loadConfig.toString();
    }
  }
}
