/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.data.shared.event;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.event.shared.HasHandlers;
import com.sencha.gxt.data.shared.event.StoreAddEvent.StoreAddHandler;

/**
 * Indicates that an element has been added the the Store, and is visible under
 * the current filter settings.
 * 
 * @param <M> the model type
 */
public final class StoreAddEvent<M> extends StoreEvent<M, StoreAddHandler<M>> {
  public interface HasStoreAddHandlers<M> extends HasHandlers {
    HandlerRegistration addStoreAddHandler(StoreAddHandler<M> handler);
  }
  public interface StoreAddHandler<M> extends EventHandler {
    void onAdd(StoreAddEvent<M> event);
  }
  private static GwtEvent.Type<StoreAddHandler<?>> TYPE;

  public static GwtEvent.Type<StoreAddHandler<?>> getType() {
    if (TYPE == null) {
      TYPE = new GwtEvent.Type<StoreAddHandler<?>>();
    }
    return TYPE;
  }

  private final List<M> items;

  private final int index;

  public StoreAddEvent(int index, List<M> items) {
    this.index = index;
    this.items = Collections.unmodifiableList(items);
  }

  @SuppressWarnings("unchecked")
  public StoreAddEvent(int index, M item) {
    this.index = index;
    items = Arrays.asList(item);
  }

  @SuppressWarnings({"unchecked", "rawtypes"})
  @Override
  public GwtEvent.Type<StoreAddHandler<M>> getAssociatedType() {
    return (GwtEvent.Type) getType();
  }

  public int getIndex() {
    return index;
  }

  public List<M> getItems() {
    return items;
  }
  @Override
  protected void dispatch(StoreAddHandler<M> handler) {
    handler.onAdd(this);
  }
}