/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.data.shared.event;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.event.shared.HasHandlers;
import com.sencha.gxt.core.client.ValueProvider;
import com.sencha.gxt.data.shared.Store;
import com.sencha.gxt.data.shared.event.StoreRecordChangeEvent.StoreRecordChangeHandler;

/**
 * Indicates that a Record object in the Store has been changed in some way.
 * 
 * @param <M> the model type
 */
public final class StoreRecordChangeEvent<M> extends StoreEvent<M, StoreRecordChangeHandler<M>> {
  public interface HasStoreRecordChangeHandlers<M> extends HasHandlers {
    HandlerRegistration addStoreRecordChangeHandler(StoreRecordChangeHandler<M> handler);
  }
  public interface StoreRecordChangeHandler<M> extends EventHandler {
    void onRecordChange(StoreRecordChangeEvent<M> event);
  }

  private static GwtEvent.Type<StoreRecordChangeHandler<?>> TYPE;

  public static Type<StoreRecordChangeHandler<?>> getType() {
    if (TYPE == null) {
      TYPE = new GwtEvent.Type<StoreRecordChangeHandler<?>>();
    }
    return TYPE;
  }

  private final Store<M>.Record record;

  private final ValueProvider<M, ?> property;// is this useful?

  public StoreRecordChangeEvent(Store<M>.Record record, ValueProvider<M, ?> property) {
    this.record = record;
    this.property = property;
  }

  @SuppressWarnings({"rawtypes", "unchecked"})
  @Override
  public GwtEvent.Type<StoreRecordChangeHandler<M>> getAssociatedType() {
    return (GwtEvent.Type) getType();
  }

  public ValueProvider<M, ?> getProperty() {
    return property;
  }

  public Store<M>.Record getRecord() {
    return record;
  }

  @Override
  protected void dispatch(StoreRecordChangeHandler<M> handler) {
    handler.onRecordChange(this);
  }
}