/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.data.shared.event;

import java.util.List;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.event.shared.HasHandlers;
import com.sencha.gxt.data.shared.Store.Record;
import com.sencha.gxt.data.shared.event.StoreUpdateEvent.StoreUpdateHandler;

/**
 * Indicates that changes to one or more {@link Record}s have been committed, so
 * the changes have affected the underlying model.
 * 
 * @param <M> the model type
 */
public final class StoreUpdateEvent<M> extends StoreEvent<M, StoreUpdateHandler<M>> {

  public interface HasStoreUpdateHandlers<M> extends HasHandlers {
    HandlerRegistration addStoreUpdateHandler(StoreUpdateHandler<M> handler);
  }

  public interface StoreUpdateHandler<M> extends EventHandler {
    void onUpdate(StoreUpdateEvent<M> event);
  }

  private static GwtEvent.Type<StoreUpdateHandler<?>> TYPE;

  public static GwtEvent.Type<StoreUpdateHandler<?>> getType() {
    if (TYPE == null) {
      TYPE = new GwtEvent.Type<StoreUpdateHandler<?>>();
    }
    return TYPE;
  }

  private List<M> items;

  public StoreUpdateEvent(List<M> items) {
    this.items = items;
  }

  @SuppressWarnings({"rawtypes", "unchecked"})
  @Override
  public GwtEvent.Type<StoreUpdateHandler<M>> getAssociatedType() {
    return (GwtEvent.Type) getType();
  }

  public List<M> getItems() {
    return items;
  }
  
  @Override
  protected void dispatch(StoreUpdateHandler<M> handler) {
    handler.onUpdate(this);
  }
}