/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.data.shared.loader;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.event.shared.HandlerRegistration;
import com.sencha.gxt.data.shared.loader.LoadExceptionEvent.LoadExceptionHandler;

/**
 * Event type for loader events.
 */
public class LoadExceptionEvent<C> extends GwtEvent<LoadExceptionHandler<C>> {

  /**
   * Handler type.
   */
  private static Type<LoadExceptionHandler<?>> TYPE;

  /**
   * Gets the type associated with this event.
   * 
   * @return returns the handler type
   */
  public static Type<LoadExceptionHandler<?>> getType() {
    return TYPE != null ? TYPE : (TYPE = new Type<LoadExceptionHandler<?>>());
  }

  private C loadConfig;
  private Throwable exception;

  public LoadExceptionEvent(C loadConfig, Throwable exception) {
    this.loadConfig = loadConfig;
    this.exception = exception;
  }

  @SuppressWarnings({"unchecked", "rawtypes"})
  @Override
  public Type<LoadExceptionHandler<C>> getAssociatedType() {
    return (Type) TYPE;
  }

  @SuppressWarnings("unchecked")
  @Override
  public Loader<?, C> getSource() {
    return (Loader<?, C>) super.getSource();
  }

  /**
   * Returns the load exception.
   * 
   * @return the load exception
   */
  public Throwable getException() {
    return exception;
  }

  /**
   * Returns the load config.
   * 
   * @return the load config
   */
  public C getLoadConfig() {
    return loadConfig;
  }

  @Override
  protected void dispatch(LoadExceptionHandler<C> handler) {
    handler.onLoadException(this);

  }
  
  /**
   * Handler class for {@link LoadExceptionEvent} events.
   */
  public interface LoadExceptionHandler<C> extends EventHandler {

    /**
     * Called when an exception occurs during a load operation.
     */
    void onLoadException(LoadExceptionEvent<C> event);
  }
  
  /**
   * A loader that implements this interface is a public source of
   * {@link LoadExceptionEvent} events.
   */
  public interface HasLoadExceptionHandlers<C> {

    /**
     * Adds a {@link LoadExceptionEvent} handler.
     * 
     * @param handler the handler
     * @return the registration for the event
     */
    public HandlerRegistration addLoadExceptionHandler(LoadExceptionHandler<C> handler);

  }

}
