/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client;

import com.google.gwt.cell.client.AbstractEditableCell;
import com.google.gwt.cell.client.ValueUpdater;
import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.EventTarget;
import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.dom.client.NodeList;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.sencha.gxt.core.client.dom.XElement;

public class ColorPaletteCell extends AbstractEditableCell<String, String> {

  public static interface ColorPaletteAppearance {

    public String getAboveColor(XElement parent, String value);

    public String getBelowColor(XElement parent, String value);

    public XElement getChildElement(XElement parent, String color);

    public String getClickedColor(XElement parent, Element target);

    public Element getColorElement(XElement parent, Element target);

    public NodeList<Element> getColorElements(XElement parent);

    public String getLeftColor(XElement parent, String value);

    public String getRightColor(XElement parent, String value);

    public void hover(XElement parent, Element target, boolean entering);

    public void onMouseOut(XElement parent, Element target, NativeEvent event);

    public void onMouseOver(XElement parent, Element target, NativeEvent event);

    public void render(Context context, String value, String[] colors, String[] labels, SafeHtmlBuilder builder);

  }

  protected ColorPaletteAppearance appearance;

  private String[] colors = new String[] {
      "E9967A", "B22222", "FFB6C1", "DB7093", "FF6347", "FFD700", "FFFACD", "FFDAB9", "BDB76B", "EE82EE", "FF00FF",
      "9400D3", "800080", "7B68EE", "7CFC00", "90EE90", "3CB371", "006400", "808000", "20B2AA", "00FFFF", "7FFFD4",
      "5F9EA0", "B0E0E6", "00BFFF", "4169E1", "000080", "FFEBCD", "DEB887", "F4A460", "D2691E", "A52A2A", "F0FFF0",
      "F0F8FF", "F5F5DC", "FFFFF0", "FFE4E1", "C0C0C0", "778899", "000000"};

  private boolean editing;

  private String[] labels = new String[] {
      "Dark Salmon", "Fire Brick", "Light Pink", "Pale Violet Red", "Tomato", "Gold", "Lemon Chiffon", "Peach Puff",
      "Dark Khaki", "Violet", "Magenta", "Dark Violet", "Purple", "Medium Slate Blue", "Lawn Green", "Light Green",
      "Medium Sea Green", "Dark Green", "Olive", "Light Sea Green", "Cyan", "Aquamarine", "Cadet Blue", "Powder Blue",
      "Deep Sky Blue", "Royal Blue", "Navy", "Blanched Almond", "Burly Wood", "Sandy Brown", "Chocolate", "Brown",
      "Honeydew", "Alice Blue", "Beige", "Ivory", "Misty Rose", "Silver", "Light Slate Gray", "Black"};

  public ColorPaletteCell() {
    this(null, null);
  }

  public ColorPaletteCell(ColorPaletteAppearance appearance) {
    this(appearance, null, null);
  }

  public ColorPaletteCell(ColorPaletteAppearance appearance, String[] colors, String[] labels) {
    super("click", "mouseover", "mouseout", "focus", "blur");
    this.appearance = appearance;
    if (colors != null) {
      this.colors = colors;
    }
    if (labels != null) {
      this.labels = labels;
    }

    // if (GXT.isAriaEnabled()) {
    // NodeList<Element> trs = getElement().select("tr");
    // for (int i = 0; i < trs.getLength(); i++) {
    // trs.getItem(i).setAttribute("role", "row");
    // }
    //
    // NodeList<Element> tds = getElement().select("td");
    // for (int i = 0; i < tds.getLength(); i++) {
    // Element elem = tds.getItem(i);
    // if (elem.getAttribute("row").equals("")) {
    // elem.setAttribute("role", "presentation");
    // }
    // }
    // getElement().setTabIndex(0);
    // getElement().setAttribute("hideFocus", "true");
    // }

    // if (GXT.isAriaEnabled()) {
    // getElement().setAttribute("role", "grid");
    // // This will need to be readded, when I know what it is suppose to do.
    // // getAriaSupport().setLabel(GXT.MESSAGES.colorPalette());
    // }
  }

  /**
   * Creates a new color palette.
   */
  public ColorPaletteCell(String[] colors, String[] labels) {
    this(GWT.<ColorPaletteAppearance> create(ColorPaletteAppearance.class), colors, labels);
  }

  /**
   * Returns the colors.
   * 
   * @return the colors
   */
  public String[] getColors() {
    return colors;
  }

  public String[] getLabels() {
    return labels;
  }

  @Override
  public boolean isEditing(Context context, Element parent, String value) {
    return editing;
  }

  @Override
  public void onBrowserEvent(Context context, Element parent, String value, NativeEvent event,
      ValueUpdater<String> valueUpdater) {
    super.onBrowserEvent(context, parent, value, event, valueUpdater);
    String type = event.getType();
    XElement xParent = parent.<XElement> cast();
    if ("click".equals(type)) {
      onClick(context, xParent, value, event, valueUpdater);
    } else if ("mouseover".equals(type)) {
      onMouseOver(context, xParent, value, event, valueUpdater);
    } else if ("mouseout".equals(type)) {
      onMouseOut(context, xParent, value, event, valueUpdater);
    } else if ("focus".equals(type)) {
      onFocus(context, xParent, value, event, valueUpdater);
    } else if ("blur".equals(type)) {
      onBlur(context, xParent, value, event, valueUpdater);
    }

  }

  @Override
  public void render(Context context, String value, SafeHtmlBuilder sb) {
    appearance.render(context, value, colors, labels, sb);
  }

  /**
   * Selects the color.
   * 
   * @param color the color
   * @param valueUpdater the value updater that provides updated data to the
   *          cell
   */
  public void select(XElement parent, Context context, String color, String value, ValueUpdater<String> valueUpdater) {
    select(parent, context, color, value, valueUpdater, false);
  }

  /**
   * Selects the color.
   * 
   * @param newValue the color
   * @param valueUpdater the value updater that provides updated data to the
   *          cell
   * @param suppressEvent true to suppress the select event
   */
  public void select(XElement parent, Context context, String newValue, String currentValue,
      ValueUpdater<String> valueUpdater, boolean suppressEvent) {
    if (!suppressEvent) {
      // if (!fireCancellableEvent(new BeforeSelectEvent())) {
      // return;
      // }
    }

    newValue = newValue.replace("#", "");

    setValue(context, parent, newValue);
    valueUpdater.update(newValue);
    XElement a = appearance.getChildElement(parent, newValue);
    parent.setAttribute("aria-activedescendant", a.getId());

    if (!suppressEvent) {
      // fireEvent(new SelectEvent());
    }
  }

  protected void onBlur(Context context, XElement parent, String currentValue, NativeEvent event,
      ValueUpdater<String> valueUpdater) {
    editing = false;
  }

  protected void onClick(Context context, XElement parent, String currentValue, NativeEvent event,
      ValueUpdater<String> valueUpdater) {
    event.preventDefault();
    // if (!disabled) {

    EventTarget from = event.getEventTarget();
    if (from != null && Element.is(from)) {
      String color = appearance.getClickedColor(parent, Element.as(from));
      if (color != null) {
        select(parent, context, color, currentValue, valueUpdater);
      }
    }
    // }
  }

  protected void onFocus(Context context, XElement parent, String currentValue, NativeEvent event,
      ValueUpdater<String> valueUpdater) {
    editing = true;
  }

  protected void onKeyDown(Context context, XElement parent, String currentValue, NativeEvent event,
      ValueUpdater<String> valueUpdater) {
    String color = appearance.getBelowColor(parent, currentValue);
    if (color != null) {
      select(parent, context, color, currentValue, valueUpdater, true);
    }
  }

  protected void onKeyLeft(Context context, XElement parent, String currentValue, NativeEvent event,
      ValueUpdater<String> valueUpdater) {
    String color = appearance.getLeftColor(parent, currentValue);
    if (color != null) {
      select(parent, context, color, currentValue, valueUpdater, true);
    }
  }

  protected void onKeyRight(Context context, XElement parent, String currentValue, NativeEvent event,
      ValueUpdater<String> valueUpdater) {
    String color = appearance.getRightColor(parent, currentValue);
    if (color != null) {
      select(parent, context, color, currentValue, valueUpdater, true);
    }
  }

  protected void onKeyUp(Context context, XElement parent, String currentValue, NativeEvent event,
      ValueUpdater<String> valueUpdater) {
    String color = appearance.getAboveColor(parent, currentValue);
    if (color != null) {
      select(parent, context, color, currentValue, valueUpdater, true);
    }
  }

  protected void onMouseOut(Context context, XElement parent, String currentValue, NativeEvent event,
      ValueUpdater<String> valueUpdater) {
    EventTarget from = event.getEventTarget();
    if (from != null && Element.is(from)) {
      appearance.onMouseOut(parent, Element.as(from), event);
    }
  }

  protected void onMouseOver(Context context, XElement parent, String currentValue, NativeEvent event,
      ValueUpdater<String> valueUpdater) {
    EventTarget from = event.getEventTarget();
    if (from != null && Element.is(from)) {
      appearance.onMouseOver(parent, Element.as(from), event);
    }
  }

}
