/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Style.Cursor;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.uibinder.client.UiChild;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.GXT;
import com.sencha.gxt.core.client.Style.Direction;
import com.sencha.gxt.core.client.dom.XDOM;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.core.client.util.Size;
import com.sencha.gxt.fx.client.animation.AfterAnimateEvent;
import com.sencha.gxt.fx.client.animation.AfterAnimateEvent.AfterAnimateHandler;
import com.sencha.gxt.fx.client.animation.Fx;
import com.sencha.gxt.fx.client.animation.SlideIn;
import com.sencha.gxt.fx.client.animation.SlideOut;
import com.sencha.gxt.messages.client.DefaultMessages;
import com.sencha.gxt.widget.core.client.Header.HeaderAppearance;
import com.sencha.gxt.widget.core.client.button.ButtonBar;
import com.sencha.gxt.widget.core.client.button.CellButtonBase;
import com.sencha.gxt.widget.core.client.button.IconButton.IconConfig;
import com.sencha.gxt.widget.core.client.button.ToolButton;
import com.sencha.gxt.widget.core.client.container.BoxLayoutContainer.BoxLayoutPack;
import com.sencha.gxt.widget.core.client.container.SimpleContainer;
import com.sencha.gxt.widget.core.client.event.BeforeCollapseEvent;
import com.sencha.gxt.widget.core.client.event.BeforeCollapseEvent.BeforeCollapseHandler;
import com.sencha.gxt.widget.core.client.event.BeforeCollapseEvent.HasBeforeCollapseHandlers;
import com.sencha.gxt.widget.core.client.event.BeforeExpandEvent;
import com.sencha.gxt.widget.core.client.event.BeforeExpandEvent.BeforeExpandHandler;
import com.sencha.gxt.widget.core.client.event.BeforeExpandEvent.HasBeforeExpandHandlers;
import com.sencha.gxt.widget.core.client.event.CollapseEvent;
import com.sencha.gxt.widget.core.client.event.CollapseEvent.CollapseHandler;
import com.sencha.gxt.widget.core.client.event.CollapseEvent.HasCollapseHandlers;
import com.sencha.gxt.widget.core.client.event.ExpandEvent;
import com.sencha.gxt.widget.core.client.event.ExpandEvent.ExpandHandler;
import com.sencha.gxt.widget.core.client.event.ExpandEvent.HasExpandHandlers;

public class ContentPanel extends SimpleContainer implements HasBeforeExpandHandlers, HasExpandHandlers,
    HasBeforeCollapseHandlers, HasCollapseHandlers, Collapsible {

  public interface ContentPanelAppearance {

    IconConfig collapseIcon();

    IconConfig expandIcon();

    XElement getBodyWrap(XElement parent);

    XElement getContentElem(XElement parent);

    XElement getFooterElem(XElement parent);

    int getFrameHeight(XElement parent);

    int getFrameWidth(XElement parent);

    HeaderAppearance getHeaderAppearance();

    XElement getHeaderElem(XElement parent);

    void onHideHeader(XElement parent, boolean hide);

    void render(SafeHtmlBuilder sb);

  }

  public interface ContentPanelMessages {

    String panelCollapse();

    String panelExpand();

  }

  // allows us to use deferred binding for panel appearance
  public interface PanelHeaderAppearance extends HeaderAppearance {

  }

  protected class DefaultContentPanelMessages implements ContentPanelMessages {

    public String panelCollapse() {
      return DefaultMessages.getMessages().panel_collapsePanel();
    }

    public String panelExpand() {
      return DefaultMessages.getMessages().panel_expandPanel();
    }

  }

  protected Header header;
  protected ButtonBar buttonBar;
  protected ContentPanelAppearance appearance;
  private ContentPanelMessages messages;

  private boolean animating;
  private boolean animCollapse = true;
  private ToolButton collapseBtn;
  private boolean collapsed, hideCollapseTool;
  protected boolean secondPassRequired;
  private boolean collapsible;
  private boolean titleCollapse;

  private boolean headerVisible = true;

  public ContentPanel() {
    this((ContentPanelAppearance) GWT.create(ContentPanelAppearance.class));
  }

  public ContentPanel(ContentPanelAppearance appearance) {
    super(true);
    this.appearance = appearance;

    setDeferHeight(true);

    SafeHtmlBuilder sb = new SafeHtmlBuilder();
    appearance.render(sb);

    setElement(XDOM.create(sb.toSafeHtml()));

    header = new Header(appearance.getHeaderAppearance());
    ComponentHelper.setParent(this, header);

    XElement headerElem = appearance.getHeaderElem(getElement());
    headerElem.appendChild(header.getElement());

    buttonBar = new ButtonBar();
    buttonBar.setVisible(false);
    appearance.getFooterElem(getElement()).appendChild(buttonBar.getElement());
  }

  @Override
  public HandlerRegistration addBeforeCollapseHandler(BeforeCollapseHandler handler) {
    return addHandler(handler, BeforeCollapseEvent.getType());
  }

  @Override
  public HandlerRegistration addBeforeExpandHandler(BeforeExpandHandler handler) {
    return addHandler(handler, BeforeExpandEvent.getType());
  }

  /**
   * Adds a button the the panel.
   * 
   * @param button the button to add
   */
  @UiChild
  public void addButton(CellButtonBase<?> button) {
    buttonBar.add(button);
  }

  /**
   * Adds a Tool to Header
   *
   * @param tool the tool to add
   */
  @UiChild
  public void addTool(Widget tool) {
    header.addTool(tool);
  }

  @Override
  public HandlerRegistration addCollapseHandler(CollapseHandler handler) {
    return addHandler(handler, CollapseEvent.getType());
  }

  @Override
  public HandlerRegistration addExpandHandler(ExpandHandler handler) {
    return addHandler(handler, ExpandEvent.getType());
  }

  /**
   * Collapses the panel body so that it becomes hidden.
   */
  public void collapse() {
    if (!collapsed && !animating && fireCancellableEvent(new BeforeCollapseEvent())) {
      hideShadow();
      onCollapse();
    }
  }

  /**
   * Expands the panel body so that it becomes visible.
   */
  public void expand() {
    if (collapsed && !animating && fireCancellableEvent(new BeforeExpandEvent())) {
      hideShadow();
      onExpand();
    }
  }

  /**
   * Returns the panel's body element.
   * 
   * @return the body
   */
  public XElement getBody() {
    return appearance.getContentElem(getElement());
  }

  /**
   * Returns the panel's button alignment.
   * 
   * @return the button alignment
   */
  public BoxLayoutPack getButtonAlign() {
    return buttonBar.getPack();
  }

  public ButtonBar getButtonBar() {
    return buttonBar;
  }

  public Header getHeader() {
    return header;
  }

  public String getHTML() {
    return header.getHTML();
  }

  public ContentPanelMessages getMessages() {
    if (messages == null) {
      messages = new DefaultContentPanelMessages();
    }
    return messages;
  }

  /**
   * Returns the minimum button width.
   * 
   * @return the minimum button width
   */
  public int getMinButtonWidth() {
    return buttonBar.getMinButtonWidth();
  }

  public String getText() {
    return header.getText();
  }

  /**
   * Returns true if animated collapsing is enabled.
   * 
   * @return true if animating
   */
  public boolean isAnimCollapse() {
    return animCollapse;
  }

  /**
   * Returns true if the panel is collapsed.
   * 
   * @return the collapsed state
   */
  public boolean isCollapsed() {
    return collapsed;
  }

  /**
   * Returns true if the panel is collapsible.
   * 
   * @return the collapsible state
   */
  public boolean isCollapsible() {
    return collapsible;
  }

  /**
   * Returns <code>true</code> if the panel is expanded.
   * 
   * @return the expand state
   */
  public boolean isExpanded() {
    return !isCollapsed();
  }

  /**
   * Returns true if the collapse tool is hidden.
   * 
   * @return the hide collapse tool state
   */
  public boolean isHideCollapseTool() {
    return hideCollapseTool;
  }

  /**
   * Returns true if title collapsing has been enabled.
   * 
   * @return true for title collapse
   */
  public boolean isTitleCollapse() {
    return titleCollapse;
  }

  @Override
  public void onBrowserEvent(Event event) {
    super.onBrowserEvent(event);
    if (event.getTypeInt() == Event.ONCLICK) {
      onClick(event);
    }
  }

  /**
   * Sets whether expand and collapse is animating (defaults to true).
   * 
   * @param animCollapse true to enable animations
   */
  public void setAnimCollapse(boolean animCollapse) {
    this.animCollapse = animCollapse;
  }

  public void setBodyBorder(boolean border) {
    appearance.getContentElem(getElement()).applyStyles(!border ? "border: 0px" : "");
  }

  public void setBodyStyle(String style) {
    appearance.getContentElem(getElement()).applyStyles(style);
  }

  public void setBodyStyleName(String style) {
    appearance.getContentElem(getElement()).addClassName(style);
  }

  /**
   * Sets the button alignment of any buttons added to this panel (defaults to
   * RIGHT, pre-render).
   * 
   * @param buttonAlign the button alignment
   */
  public void setButtonAlign(BoxLayoutPack buttonAlign) {
    assertPreRender();
    buttonBar.setPack(buttonAlign);
  }

  /**
   * True to make the panel collapsible and have the expand/collapse toggle
   * button automatically rendered into the header tool button area, false to
   * keep the panel statically sized with no button (defaults to false,
   * pre-render).
   * 
   * @param collapsible the collapsible to set
   */
  public void setCollapsible(boolean collapsible) {
    assertPreRender();
    this.collapsible = collapsible;
  }

  /**
   * Sets the panel's expand state.
   * 
   * @param expanded <code>true<code> true to expand
   */
  public void setExpanded(boolean expanded) {
    if (expanded) {
      expand();
    } else {
      collapse();
    }
  }

  public void setHeaderVisible(boolean visible) {
    appearance.onHideHeader(getElement(), !visible);
  }

  public void setHeadingHtml(SafeHtml html) {
    header.setHTML(html);
  }

  public void setHeadingHtml(String html) {
    header.setHTML(html);
  }

  /**
   * Sets the title text for the panel.
   * 
   * @param text the title text
   */
  public void setHeadingText(String text) {
    header.setText(text);
  }

  /**
   * Sets whether the collapse tool should be displayed when the panel is
   * collapsible.
   * 
   * @param hideCollapseTool true if the tool is hidden
   */
  public void setHideCollapseTool(boolean hideCollapseTool) {
    this.hideCollapseTool = hideCollapseTool;
  }

  public void setMessages(ContentPanelMessages messages) {
    this.messages = messages;
  }

  /**
   * Sets the minimum button width.
   * 
   * @param width the button width
   */
  public void setMinButtonWidth(int width) {
    buttonBar.setMinButtonWidth(width);
  }

  /**
   * True to allow expanding and collapsing the panel (when
   * {@link #setCollapsible(boolean)} = true) by clicking anywhere in the header
   * bar, false to allow it only by clicking to tool button (defaults to false).
   * 
   * @param titleCollapse the titleCollapse to set
   */
  public void setTitleCollapse(boolean titleCollapse) {
    this.titleCollapse = titleCollapse;
    if (titleCollapse) {
      header.getElement().getStyle().setCursor(Cursor.POINTER);
      sinkEvents(Event.ONCLICK);
    } else {
      header.getElement().getStyle().setCursor(Cursor.DEFAULT);
    }
  }

  protected Size adjustBodySize() {
    return new Size(0, 0);
  }

  protected void afterCollapse() {
    collapsed = true;
    animating = false;

    appearance.getBodyWrap(getElement()).hide();
    for (int i = 0; i < getWidgetCount(); i++) {
      Widget w = getWidget(i);
      if (w.isVisible() && w instanceof Component) {
        ((Component) w).notifyHide();
      }
    }

    if (buttonBar != null && buttonBar.isAttached()) {
      buttonBar.notifyHide();
    }

    sync(true);

    // Re-enable the toggle tool after an animated collapse
    if (animCollapse && collapseBtn != null) {
      collapseBtn.enable();
    }

    if (collapseBtn != null) {
      collapseBtn.changeStyle(appearance.expandIcon());
    }

    fireEvent(new CollapseEvent());
  }

  protected void afterExpand() {
    collapsed = false;
    animating = false;

    for (int i = 0; i < getWidgetCount(); i++) {
      Widget w = getWidget(i);
      if (w.isVisible() && w instanceof Component) {
        ((Component) w).notifyShow();
      }
    }

    if (buttonBar != null && buttonBar.isAttached()) {
      buttonBar.notifyShow();
    }

    sync(true);

    // Re-enable the toggle tool after an animated collapse
    if (animCollapse && collapseBtn != null) {
      collapseBtn.enable();
    }

    collapseBtn.changeStyle(appearance.collapseIcon());

    fireEvent(new ExpandEvent());
  }

  @Override
  protected void doAttachChildren() {
    super.doAttachChildren();
    ComponentHelper.doAttach(header);
    ComponentHelper.doAttach(buttonBar);
  }

  @Override
  protected void doDetachChildren() {
    super.doDetachChildren();
    ComponentHelper.doDetach(header);
    ComponentHelper.doDetach(buttonBar);
  }

  @Override
  protected XElement getContainerTarget() {
    return appearance.getContentElem(getElement());
  }

  protected Size getFrameSize() {
    return new Size(appearance.getFrameWidth(getElement()), appearance.getFrameHeight(getElement()));
  }

  protected void initTools() {
    if (collapsible && !hideCollapseTool) {
      collapseBtn = new ToolButton(appearance.collapseIcon());
      collapseBtn.addClickHandler(new ClickHandler() {
        @Override
        public void onClick(ClickEvent event) {
          event.getNativeEvent().stopPropagation();
          setExpanded(!isExpanded());
        }
      });
      header.addTool(collapseBtn);
    }
  }

  protected boolean layoutBars() {
    if (buttonBar != null && buttonBar.getWidgetCount() > 0) {

      boolean hlr = hadLayoutRunning;
      if (!hadLayoutRunning) {

        hadLayoutRunning = true;
      }
      buttonBar.forceLayout();
      hadLayoutRunning = hlr;
      sync(true);
      return true;
    }
    return false;
  }

  @Override
  protected void onAfterFirstAttach() {
    super.onAfterFirstAttach();
    if (buttonBar.getWidgetCount() > 0) {
      buttonBar.setVisible(true);
    }

    initTools();
    layoutBars();

    if (!headerVisible) {
      // appearance.onHideHeader(getElement());
      header.setVisible(false);
    }
  }

  protected void onClick(Event ce) {
    if (collapsible && titleCollapse && header.getElement().isOrHasChild(ce.getEventTarget().<Element> cast())) {
      setExpanded(!isExpanded());
    }
  }

  protected void onCollapse() {
    if (animCollapse) {
      animating = true;
      // Disable layout adjustment during animation

      // Disable toggle tool during animated collapse
      if (collapseBtn != null) {
        collapseBtn.disable();
      }

      Fx fx = new Fx(500);
      fx.addAfterAnimateHandler(new AfterAnimateHandler() {
        @Override
        public void onAfterAnimate(AfterAnimateEvent event) {
          afterCollapse();
        }
      });
      fx.run(new SlideOut(appearance.getBodyWrap(getElement()), Direction.UP));

      addStyleDependentName("animated");
    } else {
      appearance.getBodyWrap(getElement()).hide();
      afterCollapse();
    }
  }

  @Override
  protected void onDisable() {
    mask();
    super.onDisable();
  }

  @Override
  protected void onEnable() {
    unmask();
    super.onEnable();
  }

  protected void onExpand() {
    if (animCollapse) {
      animating = true;
      // Show the body before animating
      appearance.getBodyWrap(getElement()).show();

      addStyleDependentName("animated");

      Fx fx = new Fx(500);
      fx.addAfterAnimateHandler(new AfterAnimateHandler() {
        @Override
        public void onAfterAnimate(AfterAnimateEvent event) {
          afterExpand();
        }
      });
      fx.run(new SlideIn(appearance.getBodyWrap(getElement()), Direction.DOWN));

      // Disable toggle tool during animated expand
      if (collapseBtn != null) {
        collapseBtn.disable();
      }
    } else {
      appearance.getBodyWrap(getElement()).show();
      afterExpand();
    }
  }

  @Override
  protected void onResize(int width, int height) {
    Size frameSize = getFrameSize();
    Size adjustBodySize = adjustBodySize();
     
    // header image not showing when panel positioned relative in ie6
    if (GXT.isIE6()) {
      header.setWidth(header.getElement().getParentElement().getOffsetWidth());
    }

    if (isAutoWidth()) {
      getContainerTarget().getStyle().clearWidth();
    } else {
      width -= frameSize.getWidth();
      getContainerTarget().setWidth(width - adjustBodySize.getWidth(), true);

      if (buttonBar != null) {
        buttonBar.setWidth(width);
      }
    }
    layoutBars();
    if (isAutoHeight()) {
      getContainerTarget().getStyle().clearHeight();
    } else {
      height -= frameSize.getHeight();
      height -= header.getOffsetHeight();
      height -= appearance.getFooterElem(getElement()).getHeight(false);
      getContainerTarget().setHeight(height - adjustBodySize.getHeight(), true);
    }

    super.onResize(width, height);
  }
}
