/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client;

import java.util.Date;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.NodeList;
import com.google.gwt.dom.client.Style.Display;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.i18n.client.DateTimeFormat;
import com.google.gwt.i18n.client.DateTimeFormat.PredefinedFormat;
import com.google.gwt.i18n.shared.DefaultDateTimeFormatInfo;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.HasValue;
import com.sencha.gxt.core.client.GXT;
import com.sencha.gxt.core.client.Style.Direction;
import com.sencha.gxt.core.client.dom.CompositeElement;
import com.sencha.gxt.core.client.dom.CompositeFunction;
import com.sencha.gxt.core.client.dom.XDOM;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.core.client.util.DateWrapper;
import com.sencha.gxt.core.client.util.Size;
import com.sencha.gxt.core.client.util.Util;
import com.sencha.gxt.fx.client.FxElement;
import com.sencha.gxt.fx.client.animation.AfterAnimateEvent;
import com.sencha.gxt.fx.client.animation.AfterAnimateEvent.AfterAnimateHandler;
import com.sencha.gxt.fx.client.animation.Fx;
import com.sencha.gxt.messages.client.DefaultMessages;
import com.sencha.gxt.widget.core.client.button.TextButton;
import com.sencha.gxt.widget.core.client.event.SelectEvent;
import com.sencha.gxt.widget.core.client.event.SelectEvent.SelectHandler;

public class DatePicker extends Component implements HasValue<Date> {

  public interface DatePickerAppearance {

    String acitveDayClass();

    String activeDaySelector();

    String dayCellSelector();

    String dayDisabledClass();

    String dayHoverClass();

    String dayOfWeekSelector();

    String daySelector();

    String dayTextSelector();

    String leftMonthSelector();

    String leftYearSelector();

    String monthButtonSelector();

    String monthButtonTextSelector();

    String monthPickerCancelSelector();

    String monthPickerMonthSelector();

    String monthPickerOkSelector();

    String monthPickerYearSelector();

    String monthSelectedClass();

    String nextDayClass();

    String nextDaySelector();

    String prevDayClass();

    String prevDaySelector();

    void render(SafeHtmlBuilder sb);

    void renderMonthPicker(SafeHtmlBuilder sb, DatePickerMessages messages, String[] monthNames);

    String rightMonthSelector();

    String rightYearSelector();

    String selectedClass();

    String todayButtonSelector();

    String todayClass();

  }

  public class DatePickerDefaultMessages implements DatePickerMessages {

    @Override
    public String cancelText() {
      return DefaultMessages.getMessages().datePicker_cancelText();
    }

    @Override
    public String maxText() {
      return DefaultMessages.getMessages().datePicker_maxText();
    }

    @Override
    public String minText() {
      return DefaultMessages.getMessages().datePicker_minText();
    }

    @Override
    public String monthYearText() {
      return DefaultMessages.getMessages().datePicker_monthYearText();
    }

    @Override
    public String nextText() {
      return DefaultMessages.getMessages().datePicker_nextText();
    }

    @Override
    public String okText() {
      return DefaultMessages.getMessages().datePicker_okText();
    }

    @Override
    public String prevText() {
      return DefaultMessages.getMessages().datePicker_prevText();
    }

    @Override
    public String todayText() {
      return DefaultMessages.getMessages().datePicker_todayText();
    }

    @Override
    public String todayTip(String date) {
      return DefaultMessages.getMessages().datePicker_todayTip(date);
    }

  }

  public static interface DatePickerMessages {

    String cancelText();

    String maxText();

    String minText();

    String monthYearText();

    String nextText();

    String okText();

    String prevText();

    String todayText();

    String todayTip(String date);
  }

  private DateWrapper activeDate, value;
  private int firstDOW;
  private long today;
  private Date maxDate, minDate;
  private Element[] textNodes;
  private Element[] cells;
  private int startDay = Integer.MIN_VALUE;
  private DefaultDateTimeFormatInfo constants = new DefaultDateTimeFormatInfo();
  private XElement monthPicker;
  private TextButton todayBtn;
  private CompositeElement mpMonths, mpYears;
  private int mpSelMonth, mpSelYear;
  private int mpyear;
  private DatePickerMessages messages;
  private DatePickerAppearance appearance;

  public DatePicker() {
    this(GWT.<DatePickerAppearance> create(DatePickerAppearance.class));
  }

  public DatePicker(DatePickerAppearance appearance) {
    this.appearance = appearance;

    SafeHtmlBuilder builder = new SafeHtmlBuilder();
    this.appearance.render(builder);

    setElement(XDOM.create(builder.toSafeHtml()));

    setAllowTextSelection(false);

    String[] dn = constants.weekdaysNarrow();
    @SuppressWarnings("unused")
    String[] longdn = constants.weekdaysShort();

    firstDOW = getCalculatedStartDay();

    NodeList<Element> nodes = getElement().select(appearance.dayOfWeekSelector());
    for (int i = 0; i < nodes.getLength(); i++) {
      nodes.getItem(i).setInnerHTML(dn[(i + firstDOW) % 7]);
    }

    todayBtn = new TextButton(getMessages().todayText());
    todayBtn.addSelectHandler(new SelectHandler() {

      @Override
      public void onSelect(SelectEvent event) {
        selectToday();
      }
    });

    todayBtn.setToolTip(getMessages().todayTip(DateTimeFormat.getFormat(PredefinedFormat.DATE_SHORT).format(new Date())));

    getElement().selectNode(appearance.todayButtonSelector()).appendChild(todayBtn.getElement());

    monthPicker = XElement.createElement("div");
    monthPicker.getStyle().setDisplay(Display.NONE);

    getElement().appendChild(monthPicker);

    // El btntext = monthBtn.el().selectNode("button");
    // btntext.setId(XDOM.getUniqueId());
    // gridWrapper.dom.setAttribute("aria-labelledby", btntext.getId());

    cells = Util.toElementArray(getElement().select(appearance.dayCellSelector()));
    textNodes = Util.toElementArray(getElement().select(appearance.dayTextSelector()));

    activeDate = value != null ? value : new DateWrapper();
    update(activeDate);

    getElement().makePositionable();
    getElement().setTabIndex(0);
    getElement().setAttribute("hideFocus", "true");
    sinkEvents(Event.ONCLICK | Event.MOUSEEVENTS | Event.ONFOCUS);
  }

  @Override
  public HandlerRegistration addValueChangeHandler(ValueChangeHandler<Date> handler) {
    return addHandler(handler, ValueChangeEvent.getType());
  }

  /**
   * Returns the field's maximum allowed date.
   * 
   * @return the max date
   */
  public Date getMaxDate() {
    return maxDate;
  }

  /**
   * Returns the date picker messages.
   * 
   * @return the messages
   */
  public DatePickerMessages getMessages() {
    if (messages == null) {
      messages = new DatePickerDefaultMessages();
    }
    return messages;
  }

  /**
   * Returns the picker's minimum data.
   * 
   * @return the minimum date
   */
  public Date getMinDate() {
    return minDate;
  }

  /**
   * Returns the picker's start day.
   * 
   * @return the start day
   */
  public int getStartDay() {
    return startDay;
  }

  @Override
  public Date getValue() {
    return value != null ? value.asDate() : null;
  }

  @Override
  public void onBrowserEvent(Event event) {
    super.onBrowserEvent(event);
    switch (event.getTypeInt()) {
      case Event.ONCLICK:
        onClick(event);
        break;
      case Event.ONMOUSEOVER:
        onMouseOver(event);
        break;
      case Event.ONMOUSEOUT:
        onMouseOut(event);
        break;
      case Event.ONFOCUS:
        onFocus(event);
        break;
    }
  }

  /**
   * Sets the picker's maximum allowed date.
   * 
   * @param maxDate the max date
   */
  public void setMaxDate(Date maxDate) {
    if (maxDate != null) {
      maxDate = new DateWrapper(maxDate).resetTime().asDate();
    }
    this.maxDate = maxDate;
    update(value);
  }

  /**
   * Optionally, sets the date picker messages.
   * 
   * @param messages the messages
   */
  public void setMessages(DatePickerMessages messages) {
    this.messages = messages;
  }

  /**
   * Sets the picker's minimum allowed date.
   * 
   * @param minDate the minimum date
   */
  public void setMinDate(Date minDate) {
    if (minDate != null) {
      minDate = new DateWrapper(minDate).resetTime().asDate();
    }
    this.minDate = minDate;
    update(value);
  }

  /**
   * Sets the picker's start day
   * 
   * @param startDay the start day
   */
  public void setStartDay(int startDay) {
    this.startDay = startDay;
  }

  @Override
  public void setValue(Date date) {
    setValue(date, true);
  }

  @Override
  public void setValue(Date date, boolean fireEvents) {
    this.value = new DateWrapper(date).resetTime();
    update(value);
    if (fireEvents) {
      ValueChangeEvent.fire(this, date);
    }
  }

  @Override
  protected void doAttachChildren() {
    super.doAttachChildren();
    ComponentHelper.doAttach(todayBtn);
  }

  @Override
  protected void doDetachChildren() {
    super.doDetachChildren();
    ComponentHelper.doDetach(todayBtn);
  }

  protected void onClick(Event event) {
    event.preventDefault();
    XElement target = event.getEventTarget().cast();
    XElement pn = null;

    if ((pn = target.findParent(appearance.daySelector(), 2)) != null) {
      event.preventDefault();
      onDayClick(pn);
    } else if ((pn = target.findParent(appearance.leftMonthSelector(), 1)) != null) {
      showPrevMonth();
    } else if ((pn = target.findParent(appearance.rightMonthSelector(), 1)) != null) {
      showNextMonth();
    } else if ((pn = target.findParent(appearance.monthButtonSelector(), 5)) != null) {
      showMonthPicker();
    }

    if ((pn = target.findParent(appearance.monthPickerMonthSelector(), 2)) != null) {
      mpMonths.removeClassName(appearance.monthSelectedClass());
      pn.addClassName(appearance.monthSelectedClass());
      mpSelMonth = pn.getPropertyInt("xmonth");
    } else if ((pn = target.findParent(appearance.monthPickerYearSelector(), 2)) != null) {
      mpYears.removeClassName(appearance.monthSelectedClass());
      pn.addClassName(appearance.monthSelectedClass());
      mpSelYear = pn.getPropertyInt("xyear");
    } else if (target.is(appearance.monthPickerOkSelector())) {
      DateWrapper d = new DateWrapper(mpSelYear, mpSelMonth, 1);
      update(d);
      hideMonthPicker();
    } else if (target.is(appearance.monthPickerCancelSelector())) {
      hideMonthPicker();
    } else if (target.is(appearance.leftYearSelector())) {
      updateMPYear(mpyear - 10);
    } else if (target.is(appearance.rightYearSelector())) {
      updateMPYear(mpyear + 10);
    }

    if (GXT.isSafari()) {
      focus();
    }
  }

  protected void onDayClick(XElement e) {
    if (e != null) {
      String dt = e.getPropertyString("dateValue");
      if (dt != null) {
        handleDateClick(e, dt);
        return;
      }
    }
  }

  protected void onMouseOut(Event event) {
    XElement ce = event.getEventTarget().cast();

    XElement target = ce.findParent(appearance.activeDaySelector(), 3);
    if (target == null) {
      target = ce.findParent(appearance.nextDaySelector(), 3);
    }
    if (target == null) {
      target = ce.findParent(appearance.prevDaySelector(), 3);
    }
    if (target != null) {
      target.removeClassName(appearance.dayHoverClass());
    }
  }

  protected void onMouseOver(Event event) {
    XElement ce = event.getEventTarget().cast();

    XElement target = ce.findParent(appearance.activeDaySelector(), 3);
    if (target == null) {
      target = ce.findParent(appearance.nextDaySelector(), 3);
    }
    if (target == null) {
      target = ce.findParent(appearance.prevDaySelector(), 3);
    }
    if (target != null) {
      target.addClassName(appearance.dayHoverClass());
    }

  }

  private void createMonthPicker() {
    SafeHtmlBuilder builder = new SafeHtmlBuilder();
    appearance.renderMonthPicker(builder, messages, constants.monthsShort());

    monthPicker.setInnerHTML(builder.toSafeHtml().asString());

    mpMonths = new CompositeElement(Util.toElementArray(monthPicker.select(appearance.monthPickerMonthSelector())));
    mpYears = new CompositeElement(Util.toElementArray(monthPicker.select(appearance.monthPickerYearSelector())));

    mpMonths.each(new CompositeFunction() {

      public void doFunction(Element elem, CompositeElement ce, int index) {
        index += 1;
        if (index % 2 == 0) {
          elem.setPropertyInt("xmonth", (int) (5 + (Math.round(index * .5))));
        } else {
          elem.setPropertyInt("xmonth", (int) (Math.round((index - 1) * .5)));
        }
      }

    });
  }

  private XElement fly(Element elem) {
    return elem.cast();
  }

  private int getCalculatedStartDay() {
    return startDay != Integer.MIN_VALUE ? startDay : constants.firstDayOfTheWeek() - 1;
  }

  private void handleDateClick(XElement target, String dt) {
    String[] tokens = dt.split(",");
    int year = Integer.parseInt(tokens[0]);
    int month = Integer.parseInt(tokens[1]);
    int day = Integer.parseInt(tokens[2]);
    Date d = new DateWrapper(year, month, day).asDate();
    if (d != null && !target.getParentElement().<XElement> cast().hasClassName(appearance.dayDisabledClass())) {
      setValue(d);
    }
  }

  private void hideMonthPicker() {
    Fx fx = new Fx();
    fx.addAfterAnimateHandler(new AfterAnimateHandler() {
      @Override
      public void onAfterAnimate(AfterAnimateEvent event) {
        monthPicker.setVisible(false);
      }
    });
    monthPicker.<FxElement>cast().slideOut(Direction.UP, fx);
  }

  private void selectToday() {
    setValue(new DateWrapper().asDate());
  }

  private void setCellStyle(Element cell, Date d, long sel, long min, long max) {
    long t = d.getTime();

    DateWrapper w = new DateWrapper(d);
    int year = w.getFullYear();
    int month = w.getMonth();
    int day = w.getDate();

    String dd = year + "," + month + "," + day;

    cell.getFirstChildElement().setPropertyString("dateValue", dd);

    if (t == today) {
      fly(cell).addClassName(appearance.todayClass());
    }
    if (t == sel) {
      fly(cell).addClassName(appearance.selectedClass());
    } else {
      fly(cell).removeClassName(appearance.selectedClass());
    }
    if (t > max || t < min) {
      fly(cell).addClassName(appearance.dayDisabledClass());
      if (t > max) {
        cell.setTitle(messages.maxText());
      } else {
        cell.setTitle(messages.minText());
      }
    }
  }

  private void showMonthPicker() {
    createMonthPicker();

    Size s = getElement().getSize(true);
    s.setHeight(s.getHeight() - 2);
    monthPicker.setTop(1);
    monthPicker.setSize(s.getWidth(), s.getHeight());
    monthPicker.getFirstChildElement().getFirstChildElement().<XElement> cast().setSize(s.getWidth(), s.getHeight(),
        true);

    mpSelMonth = (activeDate != null ? activeDate : value).getMonth();

    updateMPMonth(mpSelMonth);
    mpSelYear = (activeDate != null ? activeDate : value).getFullYear();
    updateMPYear(mpSelYear);

    monthPicker.getStyle().setDisplay(Display.BLOCK);
    monthPicker.makePositionable(true);
    monthPicker.<FxElement>cast().slideIn(Direction.DOWN);
  }

  private void showNextMonth() {
    update(activeDate.addMonths(+1));
  }

  private void showPrevMonth() {
    update(activeDate.addMonths(-1));
  }

  private void update(DateWrapper date) {
    DateWrapper vd = activeDate;
    activeDate = date;
    if (vd != null) {
      int days = date.getDaysInMonth();
      DateWrapper firstOfMonth = date.getFirstDayOfMonth();
      int startingPos = firstOfMonth.getDayInWeek() - firstDOW;

      if (startingPos <= firstDOW) {
        startingPos += 7;
      }

      // go to previous month.
      DateWrapper pm = activeDate.addMonths(-1);
      int prevStart = pm.getDaysInMonth() - startingPos;

      days += startingPos;

      DateWrapper d = new DateWrapper(pm.getFullYear(), pm.getMonth(), prevStart).resetTime();
      today = new DateWrapper().resetTime().getTime();
      long sel = value != null ? value.resetTime().getTime() : Long.MIN_VALUE;
      long min = minDate != null ? new DateWrapper(minDate).getTime() : Long.MIN_VALUE;
      long max = maxDate != null ? new DateWrapper(maxDate).getTime() : Long.MAX_VALUE;

      int i = 0;
      for (; i < startingPos; i++) {
        textNodes[i].setInnerHTML("" + ++prevStart);
        d = d.addDays(1);
        cells[i].setClassName(appearance.prevDayClass());
        setCellStyle(cells[i], d.asDate(), sel, min, max);
      }
      for (; i < days; i++) {
        int intDay = i - startingPos + 1;
        textNodes[i].setInnerHTML("" + intDay);
        d = d.addDays(1);
        cells[i].setClassName(appearance.acitveDayClass());
        setCellStyle(cells[i], d.asDate(), sel, min, max);
      }
      int extraDays = 0;
      for (; i < 42; i++) {
        textNodes[i].setInnerHTML("" + ++extraDays);
        d = d.addDays(1);
        cells[i].setClassName(appearance.nextDayClass());
        setCellStyle(cells[i], d.asDate(), sel, min, max);
      }
      int month = activeDate.getMonth();

      String t = constants.monthsFullStandalone()[month] + " " + activeDate.getFullYear();
      getElement().selectNode(appearance.monthButtonTextSelector()).setInnerHTML(t);
    }

  }

  private void updateMPMonth(int month) {
    for (int i = 0; i < mpMonths.getCount(); i++) {
      Element elem = mpMonths.item(i);
      int xmonth = elem.getPropertyInt("xmonth");
      fly(elem).setClassName(appearance.monthSelectedClass(), xmonth == month);
    }
  }

  private void updateMPYear(int year) {
    mpyear = year;
    for (int i = 1; i <= 10; i++) {
      XElement td = XElement.as(mpYears.item(i - 1));
      int y2;
      if (i % 2 == 0) {
        y2 = (int) (year + (Math.round(i * .5)));
      } else {
        y2 = (int) (year - (5 - Math.round(i * .5)));
      }
      td.getFirstChildElement().setInnerHTML("" + y2);
      td.setPropertyInt("xyear", y2);
      td.setClassName(appearance.monthSelectedClass(), y2 == mpSelYear);
    }

  }
}
