/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client;

import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.dom.XElement;

/**
 * Creates a widget from a widget. This allows widgets to be treated as
 * components.
 */
public class WidgetComponent extends Component {

  /**
   * The wrapped widget.
   */
  protected Widget widget;

  /**
   * Creates a new widget wrapper.
   * 
   * @param widget the widget to be wrapped
   */
  public WidgetComponent(Widget widget) {
    assert widget != null : "widget must not be null";
    widget.removeFromParent();
    this.widget = widget;
    setElement(widget.getElement());
    setParent(this, widget);
  }

  @Override
  public XElement getElement() {
    return widget.getElement().<XElement> cast();
  }

  /**
   * Returns the underlying widget.
   * 
   * @return the widget
   */
  public Widget getWidget() {
    return widget;
  }
  /**
   * Determines whether this widget is currently attached to the browser's document (i.e., there is an unbroken chain of widgets between this widget and the underlying browser document). 
   * 
   * @return true if the widget is attached
   */
  @Override
  public boolean isAttached() {
    if (widget != null) {
      return widget.isAttached();
    }
    return false;
  }

  @Override
  public void onBrowserEvent(Event event) {
    // Fire any handler added to the WidgetComponent itself.
    super.onBrowserEvent(event);

    // Delegate events to the widget.
    widget.onBrowserEvent(event);
  }

  @Override
  public void setHeight(int height) {
    widget.setHeight(height + "px");
  }

  @Override
  public void setPixelSize(int width, int height) {
    widget.setPixelSize(width, height);
  }
  
  @Override
  public void setWidth(int width) {
    widget.setWidth(width + "px");
  }

  @Override
  protected void onAttach() {
    ComponentHelper.doAttach(widget);
    DOM.setEventListener(getElement(), this);
    onLoad();
  }

  @Override
  protected void onDetach() {
    try {
      onUnload();
    } finally {
      ComponentHelper.doDetach(widget);
    }
  }

  @Override
  protected void onDisable() {
    super.onDisable();
    DOM.setElementPropertyBoolean(widget.getElement(), "disabled", true);
  }

  @Override
  protected void onEnable() {
    super.onEnable();
    DOM.setElementPropertyBoolean(widget.getElement(), "disabled", false);
  }

  private native void setParent(Widget parent, Widget child) /*-{
		child.@com.google.gwt.user.client.ui.Widget::parent = parent;
  }-*/;
}
