/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.container;

import com.google.gwt.uibinder.client.UiChild;
import com.google.gwt.user.client.ui.IsWidget;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.util.Size;

public class NorthSouthContainer extends SimpleContainer implements HasNorthWidget, HasSouthWidget {

  protected Widget northWidget, southWidget;

  @Override
  public Widget getNorthWidget() {
    return northWidget;
  }

  @Override
  public Widget getSouthWidget() {
    return southWidget;
  }

  @UiChild(limit = 1, tagname = "north")
  public void setNorthWidget(IsWidget north) {
    if (this.northWidget != null) {
      remove(this.northWidget);
    }
    if (north != null) {
      this.northWidget = north.asWidget();
      insert(this.northWidget, 0);
    }
  }

  @UiChild(limit = 1, tagname = "south")
  @Override
  public void setSouthWidget(IsWidget south) {
    if (this.southWidget != null) {
      remove(this.southWidget);
    }
    if (south != null) {
      this.southWidget = south.asWidget();
      insert(this.southWidget, getWidgetCount());
    }
  }

  @Override
  protected void doLayout() {
    if (widget != null) {
      Size size = getContainerTarget().getStyleSize();

      int width = -1;

      if (!isAutoWidth()) {
        width = size.getWidth() - getLeftRightMargins(widget);

        if (northWidget != null) {
          applyLayout(northWidget, size.getWidth() - getLeftRightMargins(northWidget), -1);
        }
        if (southWidget != null) {
          applyLayout(southWidget, size.getWidth() - getLeftRightMargins(southWidget), -1);
        }
      }
      int height = -1;
      if (!isAutoHeight()) {
        height = size.getHeight() - getTopBottomMargins(widget);
        if (northWidget != null) {
          height -= northWidget.getOffsetHeight();
          height -= getTopBottomMargins(northWidget);
        }
        if (southWidget != null) {
          height -= southWidget.getOffsetHeight();
          height -= getTopBottomMargins(southWidget);
        }
      }
      applyLayout(widget, width, height);
    }
  }

  @Override
  protected void onRemove(Widget child) {
    super.onRemove(child);
    if (child == northWidget) {
      northWidget = null;
    } else if (child == southWidget) {
      southWidget = null;
    }
  }
}
