/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.container;

import com.gargoylesoftware.htmlunit.javascript.host.Event;
import com.google.gwt.core.client.Scheduler;
import com.google.gwt.dom.client.Style.Position;
import com.google.gwt.event.dom.client.HasScrollHandlers;
import com.google.gwt.event.dom.client.ScrollEvent;
import com.google.gwt.event.dom.client.ScrollHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.uibinder.client.UiChild;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.IndexedPanel;
import com.google.gwt.user.client.ui.IsWidget;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.dom.HasScrollSupport;
import com.sencha.gxt.core.client.dom.ScrollSupport;
import com.sencha.gxt.core.client.dom.XDOM;
import com.sencha.gxt.core.client.util.Margins;
import com.sencha.gxt.core.client.util.Size;

public class VerticalLayoutContainer extends InsertResizeContainer implements HasScrollHandlers, HasScrollSupport {
  public static class VerticalLayoutData extends MarginData implements HasWidth, HasHeight, LayoutData {

    private double height = -1d;
    private double width = -1d;

    public VerticalLayoutData() {

    }

    public VerticalLayoutData(double width, double height) {
      setWidth(width);
      setHeight(height);
    }

    public VerticalLayoutData(double width, double height, Margins margins) {
      super(margins);
      setWidth(width);
      setHeight(height);
    }

    public double getHeight() {
      return height;
    }

    public double getWidth() {
      return width;
    }

    public void setHeight(double height) {
      this.height = height;
    }

    public void setWidth(double width) {
      this.width = width;
    }

  }

  private boolean adjustForScroll;
  private boolean secondPassRequired;
  private ScrollSupport scrollSupport;

  public VerticalLayoutContainer() {
    setElement(DOM.createDiv());
    getElement().getStyle().setPosition(Position.RELATIVE);
  }

  @UiChild(tagname = "child")
  public void add(IsWidget child, VerticalLayoutData layoutData) {
    if (child != null) {
      child.asWidget().setLayoutData(layoutData);
    }
    super.add(child);
  }

  @Override
  public HandlerRegistration addScrollHandler(ScrollHandler handler) {
    DOM.sinkEvents(getContainerTarget(), Event.SCROLL | DOM.getEventsSunk(getContainerTarget()));
    return addDomHandler(handler, ScrollEvent.getType());
  }

  @Override
  public ScrollSupport getScrollSupport() {
    return scrollSupport;
  }

  public void insert(IsWidget w, int beforeIndex, VerticalLayoutData layoutData) {
    if (w != null) {
      w.asWidget().setLayoutData(layoutData);
    }
    super.insert(w, beforeIndex);
  }

  /**
   * Returns true if adjust for scroll is enabled.
   * 
   * @return the adjust for scroll state
   */
  public boolean isAdjustForScroll() {
    return adjustForScroll;
  }

  /**
   * True to adjust the container width calculations to account for the scroll
   * bar (defaults to false).
   * 
   * @param adjustForScroll the adjust for scroll state
   */
  public void setAdjustForScroll(boolean adjustForScroll) {
    this.adjustForScroll = adjustForScroll;
  }

  @Override
  public void setScrollSupport(ScrollSupport support) {
    this.scrollSupport = support;
  }

  @Override
  protected void doLayout() {
    Size size = getContainerTarget().getStyleSize();

    int w = size.getWidth() - (adjustForScroll ? XDOM.getScrollBarWidth() : 0);
    int h = size.getHeight();
    int ph = h;

    int count = getWidgetCount();

    // some columns can be percentages while others are fixed
    // so we need to make 2 passes
    for (int i = 0; i < count; i++) {
      Widget c = getWidget(i);
      c.getElement().getStyle().setPosition(Position.RELATIVE);
      double height = -1;
      Object d = c.getLayoutData();
      if (d instanceof HasHeight) {
        height = ((HasHeight) d).getHeight();
      }
      if (height > 1) {
        ph -= height;
      } else if (height == -1) {
        if ((c instanceof HasWidgets || c instanceof IndexedPanel) && !secondPassRequired) {
          secondPassRequired = true;
          Scheduler.get().scheduleEntry(forceLayoutCommand);
          return;
        }

        ph -= c.getOffsetHeight();
        ph -= getTopBottomMargins(c);
      } else if (height < -1) {
        ph -= (h + height);
        ph -= getTopBottomMargins(c);
      }

    }

    secondPassRequired = false;

    ph = ph < 0 ? 0 : ph;

    for (int i = 0; i < count; i++) {
      Widget c = getWidget(i);
      double width = -1;
      double height = -1;

      Object d = c.getLayoutData();
      if (d instanceof HasWidth) {
        width = ((HasWidth) d).getWidth();
      }
      if (d instanceof HasHeight) {
        height = ((HasHeight) d).getHeight();
      }

      if (width >= 0 && width <= 1) {
        width = width * w;
      } else if (width < -1) {
        width = w + width;
      }

      if (width != -1) {
        width -= getLeftRightMargins(c);
      }

      if (height >= 0 && height <= 1) {
        height = height * ph;
      } else if (height < -1) {
        height = h + height;
      }

      if (height != -1) {
        height -= getTopBottomMargins(c);
      }

      applyLayout(c, (int) Math.floor(width), (int) Math.floor(height));
    }
  }
  
}
