/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.form;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.DivElement;
import com.google.gwt.dom.client.Document;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.InputElement;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.user.client.Event;
import com.sencha.gxt.core.client.GXT;
import com.sencha.gxt.core.client.dom.XDOM;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.core.client.resources.CommonStyles;
import com.sencha.gxt.core.client.util.BaseEventPreview;
import com.sencha.gxt.core.client.util.Rectangle;
import com.sencha.gxt.messages.client.DefaultMessages;
import com.sencha.gxt.widget.core.client.Component;
import com.sencha.gxt.widget.core.client.ComponentHelper;
import com.sencha.gxt.widget.core.client.button.TextButton;

public class FileUploadField extends Component implements IsField<String> {

  public interface FileUploadFieldAppearance {

    String buttonClass();

    String fileInputClass();

    void render(SafeHtmlBuilder sb);

    String textFieldClass();

    String wrapClass();
  }

  public interface FileUploadFieldMessages {
    String browserText();
  }

  protected class DefaultFileUploadFieldMessages implements FileUploadFieldMessages {

    @Override
    public String browserText() {
      return DefaultMessages.getMessages().uploadField_browseText();
    }

  }

  private FileUploadFieldMessages messages;
  private BaseEventPreview focusPreview;
  private final FileUploadFieldAppearance appearance;
  private int buttonOffset = 3;
  private TextButton button;
  private XElement file;
  private String name;
  private TextField input;

  public FileUploadField() {
    this(GWT.<FileUploadFieldAppearance> create(FileUploadFieldAppearance.class));
  }

  public FileUploadField(FileUploadFieldAppearance appearance) {
    this.appearance = appearance;

    SafeHtmlBuilder builder = new SafeHtmlBuilder();
    this.appearance.render(builder);

    setElement(XDOM.create(builder.toSafeHtml()));

    input = new TextField();
    input.setReadOnly(true);
    getElement().appendChild(input.getElement());

    sinkEvents(Event.ONCHANGE | Event.ONCLICK | Event.MOUSEEVENTS);

    createFileInput();

    button = new TextButton(getMessages().browserText());
    DivElement wrapper = Document.get().createDivElement();
    wrapper.addClassName(appearance.buttonClass());
    XElement buttonElement = button.getElement();
    if (GXT.isIE8()) {
      buttonElement.removeClassName(CommonStyles.get().inlineBlock());
    }
    wrapper.appendChild(buttonElement);
    getElement().appendChild(wrapper);

    focusPreview = new BaseEventPreview();
    focusPreview.setAutoHide(false);
    ensureVisibilityOnSizing = true;
    setWidth(150);
  }

  @Override
  public void clear() {
    input.reset();
    createFileInput();
  }

  @Override
  public void clearInvalid() {
    // do nothing
  }

  public FileUploadFieldMessages getMessages() {
    if (messages == null) {
      messages = new DefaultFileUploadFieldMessages();
    }
    return messages;
  }

  public String getName() {
    return file.getAttribute("name");
  }

  @Override
  public String getValue() {
    return getFileInput().getValue();
  }

  public boolean isValid() {
    return true;
  }

  @Override
  public boolean isValid(boolean preventMark) {
    return true;
  }

  @Override
  public void onBrowserEvent(Event event) {
    super.onBrowserEvent(event);

    if (event.getTypeInt() == Event.ONCHANGE) {
      onChange(event);
    }

    if ((event.getTypeInt() != Event.ONCLICK) && event.getEventTarget().<Element> cast().isOrHasChild(file)) {
      button.onBrowserEvent(event);
    }
  }

  @Override
  public void reset() {
    clear();
  }

  public void setMessages(FileUploadFieldMessages messages) {
    this.messages = messages;
  }

  public void setName(String name) {
    this.name = name;
    file.<InputElement> cast().setName(name);
  }

  public void setReadOnly(boolean readonly) {
    if (button != null) {
      button.setEnabled(!readonly);
    }
    if (input != null) {
      input.setReadOnly(!readonly);
    }

  }

  @Override
  public void setValue(String value) {
    throw new UnsupportedOperationException("You cannot set the value for file upload field");
  }

  @Override
  public boolean validate(boolean preventMark) {
    return false;
  }

  protected void createFileInput() {
    if (file != null) {
      file.removeFromParent();
    }

    file = Document.get().createFileInputElement().cast();
    file.addEventsSunk(Event.ONCHANGE | Event.FOCUSEVENTS);
    file.setId(XDOM.getUniqueId());
    file.addClassName(appearance.fileInputClass());
    file.setTabIndex(-1);
    ((InputElement) file.cast()).setName(name);

    getElement().insertFirst(file);

    // file.setEnabled(isEnabled());
  }

  @Override
  protected void doAttachChildren() {
    super.doAttachChildren();
    ComponentHelper.doAttach(input);
    ComponentHelper.doAttach(button);
  }

  @Override
  protected void doDetachChildren() {
    super.doDetachChildren();
    ComponentHelper.doDetach(input);
    ComponentHelper.doDetach(button);
  }

  /**
   * Returns the file input element. You should not store a reference to this.
   * When resetting this field the file input will change.
   */
  protected InputElement getFileInput() {
    return (InputElement) file.cast();
  }

  @Override
  protected XElement getFocusEl() {
    return input.getElement();
  }

  @Override
  protected void onAfterFirstAttach() {
    super.onAfterFirstAttach();
    resizeFile();
  }

  @Override
  protected void onBlur(Event event) {
    super.onBlur(event);
    Rectangle rec = button.getElement().getBounds();
    if (rec.contains(BaseEventPreview.getLastXY())) {
      event.stopPropagation();
      event.preventDefault();
      return;
    }
    super.onBlur(event);
    focusPreview.remove();
  }

  protected void onChange(Event event) {
    if (GXT.isIE()) {
      input.setReadOnly(false);
    }
    input.setValue(getFileInput().getValue());
    if (GXT.isIE()) {
      input.setReadOnly(true);
    }
  }

  @Override
  protected void onDetach() {
    super.onDetach();
    if (focusPreview != null) {
      focusPreview.remove();
    }
  }

  @Override
  protected void onResize(int width, int height) {
    super.onResize(width, height);
    input.setWidth(width - button.getOffsetWidth() - buttonOffset);
    resizeFile();
  }

  protected void resizeFile() {
    file.setWidth(button.getOffsetWidth());
  }

}
