/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.form;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import com.google.gwt.user.client.ui.HasWidgets;
import com.google.gwt.user.client.ui.Widget;

public class FormPanelHelper {

  public static List<FieldLabel> getFieldLabels(HasWidgets container) {
    List<FieldLabel> fields = new ArrayList<FieldLabel>();
    getLabels(fields, container);
    return fields;
  }

  public static List<IsField<?>> getFields(HasWidgets container) {
    List<IsField<?>> fields = new ArrayList<IsField<?>>();
    getFields(fields, container);
    return fields;
  }
  
  /**
   * Returns true if the form is invalid.
   * 
   * @return true if all fields are valid
   */
  public static boolean isValid(HasWidgets container) {
    return isValid(container, false);
  }
  
  /**
   * Resets all field values.
   */
  public static void reset(HasWidgets container) {
    for (IsField<?> f : getFields(container)) {
      f.reset();
    }
  }

  /**
   * Returns the form's valid state by querying all child fields.
   * 
   * @param preventMark true for silent validation (no invalid event and field
   *          is not marked invalid)
   * 
   * @return true if all fields are valid
   */
  public static boolean isValid(HasWidgets container, boolean preventMark) {
    boolean valid = true;
    for (IsField<?> f : getFields(container)) {
      if (!f.isValid(preventMark)) {
        valid = false;
      }
    }
    return valid;
  }
  
  private static void getFields(List<IsField<?>> widgets, HasWidgets container) {
    Iterator<Widget> it = container.iterator();
    while (it.hasNext()) {
      Widget w = it.next();

      if (w instanceof IsField<?>) {
        widgets.add((IsField<?>) w);
      }

      if (w instanceof HasWidgets) {
        getFields(widgets, (HasWidgets) w);
      }
    }
  }

  private static void getLabels(List<FieldLabel> widgets, HasWidgets container) {
    Iterator<Widget> it = container.iterator();
    while (it.hasNext()) {
      Widget w = it.next();

      if (w instanceof FieldLabel) {
        widgets.add((FieldLabel) w);
        continue;
      }

      if (w instanceof HasWidgets) {
        getLabels(widgets, (HasWidgets) w);
      }
    }
  }
}
