/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.grid;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.Style.Overflow;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment.HorizontalAlignmentConstant;
import com.google.gwt.user.client.ui.IsWidget;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.widget.core.client.Component;
import com.sencha.gxt.widget.core.client.ComponentHelper;

/**
 * Column footer widget for Grid, which renders one to many aggregation rows.
 */
public class ColumnFooter<M> extends Component {

  public class Foot extends SimplePanel {

    public Foot(AggregationRowConfig<M> config) {
      setStyleName("x-grid3-cell-inner");
    }

    public void setHtml(SafeHtml html) {
      setWidget(new HTML(html));
    }
  }

  public class FooterRow extends Component {

    protected AggregationRowConfig<M> config;
    protected FlexTable table;

    public FooterRow(AggregationRowConfig<M> config) {
      this.config = config;
      rows.add(this);

      setElement(DOM.createDiv());
      setStyleName("x-grid3-footer-row");

      table = new FlexTable();
      table.setStyleName("x-grid3-row-table");
      table.setCellPadding(0);
      table.setCellSpacing(0);

      int cols = cm.getColumnCount();
      for (int i = 0; i < cols; i++) {
        Foot f = new Foot(config);

        String cellStyle = config.getCellStyle(null);
        if (cellStyle == null) {
          cellStyle = "";
        }
        table.setWidget(0, i, f);
        table.getCellFormatter().getElement(0, i).setPropertyInt("gridColumnIndex", i);
        table.getCellFormatter().setStyleName(0, i, "x-grid3-cell x-grid3-footer-cell " + cellStyle);
        HorizontalAlignmentConstant align = cm.getColumnAlignment(i);

        if (align != null) {
          table.getCellFormatter().setHorizontalAlignment(0, i, align);
        }

        if (cm.isHidden(i)) {
          updateColumnHidden(i, true);
        }
      }
      getElement().appendChild(table.getElement());
    }

    @SuppressWarnings("unchecked")
    public void setHtml(int column, SafeHtml html) {
      Foot f = (Foot) table.getWidget(0, column);
      f.setHtml(html);
    }

    @SuppressWarnings("unchecked")
    public void setWidget(int column, IsWidget widget) {
      Foot f = (Foot) table.getWidget(0, column);
      f.setWidget(widget);
    }

    @Override
    protected void doAttachChildren() {
      super.doAttachChildren();
      ComponentHelper.doAttach(table);
    }

    @Override
    protected void doDetachChildren() {
      super.doDetachChildren();
      ComponentHelper.doDetach(table);
    }

  }

  protected ColumnModel<M> cm;
  protected Grid<M> grid;
  protected List<FooterRow> rows = new ArrayList<FooterRow>();

  public ColumnFooter(Grid<M> grid, ColumnModel<M> cm) {
    this.grid = grid;
    this.cm = cm;

    setElement(DOM.createDiv());
    setStyleName("x-grid3-footer");
    getElement().getStyle().setOverflow(Overflow.HIDDEN);

    int rows = cm.getAggregationRows().size();
    for (int i = 0; i < rows; i++) {
      FooterRow row = new FooterRow(cm.getAggregationRow(i));
      getElement().appendChild(row.getElement());
    }
  }

  public void add(FooterRow row) {
    rows.add(row);
  }

  public void remove(FooterRow row) {
    rows.remove(row);
  }

  public void updateColumnHidden(int column, boolean hidden) {
    for (int i = 0; i < rows.size(); i++) {
      FooterRow footerRow = rows.get(i);
      Element td = footerRow.table.getFlexCellFormatter().getElement(0, column);
      td.getStyle().setProperty("display", hidden ? "none" : "");
    }
  }

  public void updateColumnWidth(int column, int width) {
    if (!cm.isHidden(column)) {
      for (int i = 0; i < rows.size(); i++) {
        FooterRow row = rows.get(i);
        XElement.as(row.table.getCellFormatter().getElement(0, column)).setWidth(width, true);
        Widget w = row.table.getWidget(0, column);
        XElement.as(w.getElement()).setWidth(width - 2, true);
      }
    }
  }

  public void updateTotalWidth(int offset, int width) {
    for (int i = 0; i < rows.size(); i++) {
      FooterRow row = rows.get(i);
      row.setWidth(offset);
      row.table.getElement().getStyle().setPropertyPx("width", width);
    }
  }

  @Override
  protected void doAttachChildren() {
    super.doAttachChildren();
    for (FooterRow row : rows) {
      ComponentHelper.doAttach(row);
    }
  }

  @Override
  protected void doDetachChildren() {
    super.doAttachChildren();
    for (FooterRow row : rows) {
      ComponentHelper.doDetach(row);
    }
  }

  @Override
  protected void onAfterFirstAttach() {
    refresh();
    super.onAfterFirstAttach();
  }

  protected void refresh() {
    int cols = cm.getColumnCount();
    int rowcount = rows.size();

    for (int i = 0; i < rowcount; i++) {
      for (int j = 0; j < cols; j++) {
        refreshCol(i, j);
      }
    }
  }

  protected <N, O> void refreshCol(int row, int col) {
    FooterRow footer = rows.get(row);
    AggregationRowConfig<M> config = cm.getAggregationRow(row);
    ColumnConfig<M, N> c = cm.getColumn(col);
    updateColumnWidth(col, cm.getColumnWidth(col));

    AggregationRenderer<M> renderer = config.getRenderer(c);
    if (renderer != null) {
      SafeHtml s = renderer.render(col, grid);
      if (s != null) {
        footer.setHtml(col, s);
        return;
      }
      IsWidget w = renderer.renderIsWidget(col, grid);
      if (w != null) {
        footer.setWidget(col, w);
        return;
      }
    }

  }
}
