/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.grid;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Stack;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.NodeList;
import com.google.gwt.dom.client.TableSectionElement;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.sencha.gxt.core.client.ValueProvider;
import com.sencha.gxt.core.client.XTemplates;
import com.sencha.gxt.core.client.dom.XDOM;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.data.shared.ListStore;

public class GroupSummaryView<M> extends GroupingView<M> {

  public interface CellTemplate {
    SafeHtml renderCell(String id, String classNames, String style, Object value);
  }

  public interface DefaultCellTemplate extends XTemplates, CellTemplate {
    @Override
    @XTemplate(source = "GroupSummaryCell.html")
    public SafeHtml renderCell(String id, String classNames, String style, Object value);
  }

  public interface DefaultRowTemplate extends XTemplates, RowTemplate {
    @Override
    @XTemplate(source = "GroupSummaryRow.html")
    public SafeHtml renderRow(String style, SafeHtml cells);
  }

  public interface RowTemplate {
    SafeHtml renderRow(String style, SafeHtml cells);
  }

  protected RowTemplate rowTemplate = GWT.create(DefaultRowTemplate.class);
  protected CellTemplate cellTemplate = GWT.create(DefaultCellTemplate.class);

  /**
   * Returns the summary node element.
   * 
   * @param g the group element
   * @return the summary node
   */
  public XElement getSummaryNode(Element g) {
    if (g != null) {
      return fly(g).down(".x-grid3-summary-row");
    }
    return null;
  }

  /**
   * Returns true if summaries are visible.
   * 
   * @return true for visible
   */
  public boolean isSummaryVisible() {
    return !grid.getElement().hasClassName("x-grid-hide-summary");
  }

  /**
   * Toggles the summary information visibility.
   * 
   * @param visible true for visible, false to hide
   */
  public void toggleSummaries(boolean visible) {
    XElement el = grid.getElement();
    if (el != null) {
      if (visible) {
        el.removeClassName("x-grid-hide-summary");
      } else {
        el.addClassName("x-grid-hide-summary");
      }
    }
  }

  @SuppressWarnings({"unchecked", "rawtypes"})
  protected Map<ValueProvider<M, ?>, Number> calculate(List<M> models) {
    Map<ValueProvider<M, ?>, Number> data = new HashMap<ValueProvider<M, ?>, Number>();

    for (int i = 0, len = cm.getColumnCount(); i < len; i++) {
      SummaryColumnConfig cf = (SummaryColumnConfig) cm.getColumn(i);
      if (cf.getSummaryType() != null) {
        data.put(cf.getValueProvider(), (Number) cf.getSummaryType().calculate(models, cf.getValueProvider()));
      }
    }
    return data;
  }

  protected void doAllWidths(List<Integer> ws, int tw) {
    if (!enableGrouping) return;
    NodeList<Element> gs = getGroups();
    for (int i = 0, len = gs.getLength(); i < len; i++) {
      XElement s = gs.getItem(i).getChildNodes().getItem(2).cast();
      s.setWidth(tw);
      if (s.getFirstChild() == null) return;
      s.getFirstChildElement().<XElement> cast().setWidth(tw);
      TableSectionElement tse = s.getFirstChildElement().cast();
      NodeList<Element> cells = tse.getRows().getItem(0).getChildNodes().cast();
      for (int j = 0, wlen = ws.size(); j < wlen; j++) {
        cells.getItem(j).<XElement> cast().setWidth(ws.get(j));
      }
    }
  }

  protected void doWidth(int col, int w, int tw) {
    if (!enableGrouping) return;
    NodeList<Element> gs = getGroups();
    for (int i = 0, len = gs.getLength(); i < len; i++) {
      XElement s = gs.getItem(i).getChildNodes().getItem(2).cast();
      s.setWidth(tw);
      s.getFirstChildElement().<XElement> cast().setWidth(tw);

      TableSectionElement tse = s.getFirstChildElement().cast();
      XElement e = tse.getRows().getItem(0).getChildNodes().getItem(col).cast();
      e.setWidth(w);
    }
  }

  protected void onRemove(M m, int index, boolean isUpdate) {
    super.onRemove(m, index, isUpdate);
    if (!isUpdate) {
      refreshSummaries();
    }
  }

  @Override
  protected void onUpdate(ListStore<M> store, List<M> models) {
    super.onUpdate(store, models);
    refreshSummaries();
  }

  protected void refreshSummaries() {
    NodeList<Element> groups = getGroups();
    List<GroupingData<M>> groupData = getGroupData();
    for (int i = 0; i < groupData.size(); i++) {
      Element g = groups.getItem(i);
      if (g == null) continue;
      SafeHtml s = renderGroupSummary(groupData.get(i));
      XElement existing = getSummaryNode(g);
      if (existing != null) {
        existing.removeFromParent();
      }
      g.appendChild(XDOM.create(s));
    }
  }

  @Override
  protected SafeHtml renderGroupSummary(GroupingData<M> groupInfo) {
    Map<ValueProvider<M, ?>, Number> data = calculate(groupInfo.getItems());
    return renderSummary(groupInfo, data);
  };

  @SuppressWarnings({"rawtypes", "unchecked"})
  protected SafeHtml renderSummary(GroupingData<M> groupInfo, Map<ValueProvider<M, ?>, Number> data) {
    SafeHtmlBuilder b = new SafeHtmlBuilder();

    int last = cm.getColumnCount() - 1;
    for (int i = 0, len = cm.getColumnCount(); i < len; i++) {
      SummaryColumnConfig cf = (SummaryColumnConfig) cm.getColumn(i);

      ColumnData c = getColumnData().get(i);
      String id = cf.getColumnClassName() != null ? cf.getColumnClassName() : "";
      String style = c.style;
      String css = i == 0 ? "x-grid3-cell-first " : (i == last ? "x-grid3-cell-last" : "");

      Number n = data.get(cm.getValueProvider(i));
      String value = "";

      if (cf.getSummaryFormat() != null) {
        value = n == null ? "" : cf.getSummaryFormat().format(n.doubleValue());
      } else if (cf.getSummaryRenderer() != null) {
        value = cf.getSummaryRenderer().render(n, data).asString();
      } else {

      }

      b.append(cellTemplate.renderCell(id, css, style, value));
    }

    SafeHtml cells = b.toSafeHtml();

    String tstyle = "width:" + getTotalWidth() + "px;";

    b = new SafeHtmlBuilder();
    b.append(rowTemplate.renderRow(tstyle, cells));
    return b.toSafeHtml();
  }

  @Override
  protected void templateOnAllColumnWidthsUpdated(Stack<Integer> ws, int tw) {
    super.templateOnAllColumnWidthsUpdated(ws, tw);
    doAllWidths(ws, tw);
  }
  
  @Override
  protected void templateOnColumnWidthUpdated(int col, int w, int tw) {
    super.templateOnColumnWidthUpdated(col, w, tw);
    doWidth(col, w, tw);
  }
  
  @Override
  protected void templateOnColumnHiddenUpdated(int col, boolean hidden, int tw) {
    super.templateOnColumnHiddenUpdated(col, hidden, tw);
    if (!enableGrouping) return;
    NodeList<Element> gs = getGroups();
    String display = hidden ? "none" : "";
    for (int i = 0, len = gs.getLength(); i < len; i++) {
      Element s = gs.getItem(i).getChildNodes().getItem(2).cast();
      s.getStyle().setPropertyPx("width", tw);
      s.getFirstChildElement().getStyle().setPropertyPx("width", tw);
      TableSectionElement tse = s.getFirstChildElement().cast();
      Element e = tse.getRows().getItem(0).getChildNodes().getItem(col).cast();
      e.getStyle().setProperty("display", display);
    }
  }
}
