/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.grid;

import java.util.List;

import com.sencha.gxt.core.client.ValueProvider;

/**
 * Calculates the value of a summary column.
 * 
 * @param <M> the model type
 * @param <N> the value type
 * @param <O> the return type
 */
public interface SummaryType<M, N, O> {

  public static final class AvgSummaryType<M, N> implements SummaryType<M, N, Double> {
    @Override
    public Double calculate(List<M> l, ValueProvider<M, N> valueProvider) {
      double avg = 0d;
      for (int i = 0, len = l.size(); i < len; i++) {
        N d = valueProvider.getValue(l.get(i));
        if (d instanceof Number) {
          avg += ((Number) d).doubleValue();
        }
      }
      return avg / l.size();
    }
  }

  public static final class CountSummaryType implements SummaryType<Object, Object, Integer> {
    @Override
    public Integer calculate(List<Object> l, ValueProvider<Object, Object> valueProvider) {
      return l.size();
    }
  }

  public static final class MaxSummaryType<M, N> implements SummaryType<M, N, Double> {
    @Override
    public Double calculate(List<M> l, ValueProvider<M, N> valueProvider) {
      Double max = null;
      for (int i = 0, len = l.size(); i < len; i++) {
        N d = valueProvider.getValue(l.get(i));
        if (d instanceof Number) {
          if (max == null) {
            max = ((Number) d).doubleValue();
          } else {
            max = Math.max(((Number) d).doubleValue(), max);
          }
        }
      }
      return max == null ? 0d : max;

    }
  }

  public static final class MinSummaryType<M, N> implements SummaryType<M, N, Double> {
    @Override
    public Double calculate(List<M> l, ValueProvider<M, N> valueProvider) {
      Double min = null;
      for (int i = 0, len = l.size(); i < len; i++) {
        N d = valueProvider.getValue(l.get(i));
        if (d instanceof Number) {
          if (min == null) {
            min = ((Number) d).doubleValue();
          } else {
            min = Math.min(((Number) d).doubleValue(), min);
          }
        }
      }
      return min == null ? 0d : min;
    }
  }

  public static final class SumSummaryType<M, N> implements SummaryType<M, N, Double> {
    @Override
    public Double calculate(List<M> l, ValueProvider<M, N> valueProvider) {
      Double sum = 0d;
      for (int i = 0, len = l.size(); i < len; i++) {
        N d = valueProvider.getValue(l.get(i));
        if (d instanceof Number) {
          sum += ((Number) d).doubleValue();
        }
      }
      return sum;
    }
  }

  /**
   * Returns the value for a summary calculation.
   * 
   * @param m the list of models
   * @param valueProvider the value provider
   * @return the summary value
   */
  public abstract O calculate(List<M> m, ValueProvider<M, N> valueProvider);

}
