/**
 * Ext GWT 3.0.0-beta1 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.selection;

import java.util.List;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.event.shared.HandlerRegistration;
import com.sencha.gxt.widget.core.client.Component;
import com.sencha.gxt.widget.core.client.selection.SelectionChangedEvent.SelectionChangedHandler;

/**
 * Fires before the selection changes.
 */
public class SelectionChangedEvent<M> extends GwtEvent<SelectionChangedHandler<M>> {

  /**
   * Handler type.
   */
  private static Type<SelectionChangedHandler<?>> TYPE;

  /**
   * Gets the type associated with this event.
   * 
   * @return returns the handler type
   */
  public static Type<SelectionChangedHandler<?>> getType() {
    if (TYPE == null) {
      TYPE = new Type<SelectionChangedHandler<?>>();
    }
    return TYPE;
  }

  private List<M> selection;

  public SelectionChangedEvent(List<M> selection) {
    this.selection = selection;
  }

  public List<M> getSelection() {
    return selection;
  }

  @SuppressWarnings({"unchecked", "rawtypes"})
  @Override
  public Type<SelectionChangedHandler<M>> getAssociatedType() {
    return (Type) TYPE;
  }

  public Component getSource() {
    return (Component) super.getSource();
  }

  @Override
  protected void dispatch(SelectionChangedHandler<M> handler) {
    handler.onSelectionChanged(this);
  }
  
  /**
   * Handler class for {@link SelectionChangedEvent} events.
   */
  public interface SelectionChangedHandler<M> extends EventHandler {

    /**
     * Called before a widget's selections are changed. 
     */
    void onSelectionChanged(SelectionChangedEvent<M> event);
  }
  
  /**
   * A widget that implements this interface is a public source of
   * {@link SelectionChangedEvent} events.
   * 
   * @param <M> the model type
   */
  public interface HasSelectionChangedHandlers<M> {

    /**
     * Adds a {@link SelectionChangedHandler} handler for
     * {@link SelectionChangedEvent} events.
     * 
     * @param handler the handler
     * @return the registration for the event
     */
    HandlerRegistration addSelectionChangedHandler(SelectionChangedHandler<M> handler);
  }

}
